# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin
from django.utils.translation import gettext_lazy as _
from django.urls import path

from jmb.filters.admin import AdvancedSearchModelAdmin
from thx.core.admin import UserAdminMixin
from thx.admintools.object_tools import ObjectToolsMixin

from . import models
from .utils import AsyncRunJobs


class JobLogInline(UserAdminMixin, admin.TabularInline):
    model = models.JobLog
    extra = 1
    classes = ('collapse',)
    readonly_fields = ('date_create', 'elaboration_results', 'is_valid', 'extra_info',)
    exclude = ('elaboration_time', 'date_last_modify', 'creator', 'last_modifier', 'log_file_content',)


class JobAdmin(
    ObjectToolsMixin, UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    advanced_search_fields = (
        ('name__icontains', 'command__icontains',),
        ('year__gte', 'year__lte', 'year__exact',),
        ('month__gte', 'month__lte', 'month__exact',),
        ('day__gte', 'day__lte', 'day__exact',),
        ('weekday__gte', 'weekday__lte', 'weekday__exact',),
        ('hour__gte', 'hour__lte', 'hour__exact',),
        ('minute__gte', 'minute__lte', 'minute__exact',),
    )
    search_fields = ('name',)
    list_filter = ('status', 'is_una_tantum', 'command',)
    fieldsets = (
        (_('general information'), {
            'fields': (
                ('year', 'month', 'day', 'weekday',),
                ('hour', 'minute',),
                'name',
                'command',
                'parameters',
                'status',
                'is_una_tantum'
            )
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ('ordering', )
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify')
        }),
    )
    list_display = (
        'status', 'year', 'month', 'day', 'hour', 'minute', 'weekday', 'name', 'get_last_execution_status',
        'ordering', 'is_una_tantum'
    )
    readonly_fields = ('creator', 'date_create', 'last_modifier', 'date_last_modify')
    inlines = (JobLogInline,)

    def get_object_tools(self, request):
        return [
            {'admin_urlname': 'run_jobs', 'title': _('Run jobs')},
        ]

    def get_urls(self):
        urls = super().get_urls()
        my_urls = [
            path(
                'run-jobs/',
                admin.site.admin_view(self.run_jobs),
                name='cron_job_run_jobs'
            ),
        ]
        return my_urls + urls

    def run_jobs(self, request):
        AsyncRunJobs().start()
        return self.changelist_view(request)

    def get_last_execution_status(self, obj):
        return obj.last_execution_status
    get_last_execution_status.boolean = True
    get_last_execution_status.short_description =_('last execution status')

class JobLogAdmin(
    UserAdminMixin, AdvancedSearchModelAdmin, admin.ModelAdmin
):
    advanced_search_fields = ()
    fieldsets = (
        (_('general information'), {
            'fields': (
                'job', 'elaboration_time', 'elaboration_results', 'is_valid', 'extra_info', 'log_file_content',
            )
        }),
        (_('visualization admin'), {
            'classes': ('collapse',),
            'fields': ()
        }),
        (_('logs admin'), {
            'classes': ('collapse',),
            'fields': ('date_create', 'date_last_modify')
        }),
    )
    list_display = (
        'job', 'elaboration_time', 'elaboration_results', 'is_valid',
    )
    readonly_fields = (
        'date_create', 'date_last_modify',
        'job', 'elaboration_time', 'elaboration_results', 'is_valid', 'extra_info', 'log_file_content',
    )


admin.site.register(models.Job, JobAdmin)
admin.site.register(models.JobLog, JobLogAdmin)
