# -*- coding: utf-8 -*-
import operator
import django

from django.conf import settings

if django.VERSION < (3, 1):
    from django.contrib.postgres.fields import JSONField
else:
    from django.db.models import JSONField

from django.core import validators
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import Q
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from functools import reduce

from thx.core.models import UserModel, DateModel, OrderedModel, CleanModel

from . import utils
from .validators import validate_year


class Job(CleanModel, UserModel, DateModel, OrderedModel):
    status = models.IntegerField(
        choices=settings.CRON_JOB_STATUS_CHOICES,
        default=0,
        verbose_name=_('status'),
    )
    year = models.PositiveIntegerField(
        blank=True, null=True,
        validators=[validate_year],
        verbose_name=_('year'),
    )
    month = models.PositiveIntegerField(
        blank=True, null=True,
        validators=[validators.MinValueValidator(1), validators.MaxValueValidator(12)],
        verbose_name=_('month'),
    )
    day = models.PositiveIntegerField(
        blank=True, null=True,
        verbose_name=_('day'),
    )
    hour = models.PositiveIntegerField(
        blank=True, null=True,
        validators=[validators.MinValueValidator(0), validators.MaxValueValidator(23)],
        verbose_name=_('hour'),
    )
    minute = models.PositiveIntegerField(
        blank=True, null=True,
        validators=[validators.MinValueValidator(0), validators.MaxValueValidator(59)],
        verbose_name=_('minute'),
    )
    weekday = models.PositiveIntegerField(
        blank=True, null=True,
        validators=[validators.MinValueValidator(0), validators.MaxValueValidator(6)],
        verbose_name=_('weekday')
    )
    name = models.CharField(
        max_length=255,
        verbose_name=_('name'),
    )
    command = models.CharField(
        choices=utils.get_command(),
        max_length=255,
        verbose_name=_('command'),
    )
    parameters = JSONField(
        blank=True,
        default=dict,
        verbose_name=_('parameters'),
    )
    is_una_tantum = models.BooleanField(
        default=False,
        verbose_name=_('is una tantum'),
    )

    class Meta:
        verbose_name = _('job')
        verbose_name_plural = _('jobs')
        ordering = ('ordering', 'name',)
        permissions = (
            ("list_job", "Can list job"),
            ("detail_job", "Can detail job"),
            ("disable_job", "Can disable job"),
        )

    def __str__(self):
        return self.name

    def clean(self):
        self.check_date()
        self.check_parameters()
        # super().clean()

    def check_date(self):
        if self.year and self.month and self.day:
            # TODO: fix datetime with tmz
            try:
                timezone.datetime(self.year, self.month, self.day)
            except:
                raise ValidationError(_("insert valid year, month and day"))

        elif self.year is None and self.month and self.day:
            try:
                timezone.datetime(2000, self.month, self.day)
            except:
                raise ValidationError(_("insert valid month and day"))

    def check_parameters(self):
        if self.parameters == None:
            raise ValidationError(_("value must be valid JSON"))

    @property
    def is_ripetitive(self):
        if self.is_una_tantum:
            return False
        return not all(v is not None for v in [self.year, self.month, self.day, self.hour, self.minute])

    @property
    def last_execution_status(self):
        last_job_execution = self.job_elaborations.all().order_by('date_create').last()
        if last_job_execution:
            return last_job_execution.is_valid

    def set_running(self, info=None):
        running = -1
        self.status = running
        self.save()

    def set_completed(self, info=None):
        completed = 1
        self.status = completed
        self.save()

    def set_to_run(self, info=None):
        to_run = 0
        self.status = to_run
        self.save()

    def set_error(self, info=None):
        completed = 1
        self.status = completed
        self.save()

    def run_command(self):
        from django.utils.module_loading import import_string
        try:
            command = import_string(self.command)
            result = command(instance=self, **self.parameters).do()
            return result
        except:
            raise

    @classmethod
    def selected_jobs(cls, datetime=None, status=0):
        if datetime is None:
            datetime = timezone.now()
        qs = cls.objects.filter(status=status)
        query1 = Q(is_una_tantum=True)
        query2 = Q(year=datetime.year) | Q(year__isnull=True)
        query2 &= Q(month=datetime.month) | Q(month__isnull=True)
        query2 &= Q(day=datetime.day) | Q(day__isnull=True)
        query2 &= Q(hour=datetime.hour) | Q(hour__isnull=True)
        query2 &= Q(minute=datetime.minute) | Q(minute__isnull=True)
        query2 &= Q(weekday=datetime.weekday()) | Q(weekday__isnull=True)
        qs = qs.filter(query1 | query2)

        # TODO test
        query_list = [
            Q(year=datetime.year) | Q(year__isnull=True),
            Q(month=datetime.month) | Q(month__isnull=True),
            Q(day=datetime.day) | Q(day__isnull=True),
            Q(hour=datetime.hour) | Q(hour__isnull=True),
            Q(minute=datetime.minute) | Q(minute__isnull=True),
            Q(weekday=datetime.weekday()) | Q(weekday__isnull=True)
        ]
        qs = qs.filter(query1 | reduce(operator.and_, query_list))

        return qs

    @classmethod
    def complete_old_jobs(cls, datetime=None):
        if datetime is None:
            datetime = timezone.now()
        to_run = 0
        completed = 1
        count = 0
        qs = cls.objects.filter(status=to_run, is_una_tantum=False)
        count += qs.filter(year__lt=datetime.year).update(status=completed)
        count += qs.filter(year=datetime.year, month__lt=datetime.month).update(status=completed)
        count += qs.filter(year=datetime.year, month=datetime.month, day__lt=datetime.day).update(status=completed)
        count += qs.filter(
            year=datetime.year, month=datetime.month, day=datetime.day,
            hour__lt=datetime.hour
        ).update(status=completed)
        count += qs.filter(
            year=datetime.year, month=datetime.month, day=datetime.day,
            hour=datetime.hour, minute__lt=datetime.minute
        ).update(status=completed)
        return count


class JobLog(DateModel, CleanModel):
    job = models.ForeignKey(
        "Job",
        on_delete=models.CASCADE,
        related_name='job_elaborations',
        verbose_name=_('job'),
    )
    elaboration_time = models.DurationField(
        null=True, blank=True,
        verbose_name=_("elaboration time"),
    )
    elaboration_results = JSONField(
        verbose_name=_('elaboration results'),
    )
    is_valid = models.BooleanField(
        blank=True, null=True,
        verbose_name=_('is valid'),
    )
    extra_info = JSONField(
        blank=True, null=True,
        verbose_name=_('extra info')
    )
    log_file_content = models.TextField(
        blank=True, null=True,
        verbose_name=_('log file content')
    )

    class Meta:
        verbose_name = _('job log')
        verbose_name_plural = _('job logs')
        ordering = ('-date_create',)
        permissions = (
            ("list_joblog", "Can list job log"),
            ("detail_joblog", "Can detail job log"),
            ("disable_joblog", "Can disable job log"),
        )

    def __str__(self):
        return str(self.date_create)

    def clean(self):
        pass
